@@ -1,3 +1,27 @@
+package Petal::CodeGenerator::Decode;
+use strict;
+use warnings;
+use base qw /MKDoc::XML::Decode/;
+
+
+##
+# $class->process ($xml);
+# -----------------------
+# Expands the entities &apos; &quot; &gt; &lt; and &amp;
+# into their ascii equivalents.
+##
+sub process
+{
+    (@_ == 2) or warn "MKDoc::XML::Encode::process() should be called with two arguments";
+    
+    my $self = shift;
+    my $data = join '', @_;
+    $data    =~ s/&(#?[0-9A-Za-z]+)\\;/$self->entity_to_char ($1)/eg;
+    
+    return $data;
+}
+
+
 # ------------------------------------------------------------------
 # Petal::CodeGenerator - Generates Perl code from canonical syntax
 # ------------------------------------------------------------------
@@ -7,7 +31,7 @@
 # code that can be turned into a subroutine using eval().
 # ------------------------------------------------------------------
 package Petal::CodeGenerator;
-use Petal::XML_Encode_Decode;
+use MKDoc::XML::Decode;
 use strict;
 use warnings;
 use Carp;
@@ -138,7 +162,7 @@ sub process
 	    %token_hash = (@atts1, @atts2); 
 	    foreach my $key (%token_hash)
 	    {
-		$token_hash{$key} = Petal::XML_Encode_Decode::decode_backslash_semicolon ($token_hash{$key})
+		$token_hash{$key} = $class->_decode_backslash_semicolon ($token_hash{$key})
 		    if (defined $token_hash{$key});
 	    }
 	    
@@ -170,12 +194,12 @@ sub process
 	}
 	else
 	{
-            my $string = $token;
-            $string =~ s/\@/\\\@/gsm;
-            $string =~ s/\$/\\\$/gsm;
-            $string =~ s/\n/\\n/gsm;
-            $string =~ s/\n//gsm;
-            $string =~ s/\"/\\\"/gsm;
+            my $string = quotemeta ($token);
+#            $string =~ s/\@/\\\@/gsm;
+#            $string =~ s/\$/\\\$/gsm;
+#            $string =~ s/\n/\\n/gsm;
+#            $string =~ s/\n//gsm;
+#            $string =~ s/\"/\\\"/gsm;
             $class->add_code($class->_add_res( '"' . $string . '";'));
         }
     }
@@ -203,12 +227,7 @@ sub _include
 	$class->add_code ("my \$res = eval { Petal->new ('$path')->process (\$hash->new()) };");
     
     $class->add_code ("\$res = \"<!--\\n\$\@\\n-->\" if (defined \$\@ and \$\@);");
-    $class->add_code ("if (scalar keys \%Encode::) {");
-    $class->indent_increment();
-    $class->add_code ("\$res = Encode::decode (\$Petal::ENCODE_CHARSET, \$res) if (\$Petal::ENCODE_CHARSET);");
-    $class->add_code ("Encode::_utf8_on (\$res) if (Encode->can ('_utf8_on'));");
-    $class->indent_decrement();
-    $class->add_code ("}");
+    # $class->add_code ("if (\$] > 5.007) { \$res = Petal->_decode_for_codegenerator (\$res) }");
     $class->add_code ("\$res;");
     $class->indent_decrement();
     $class->add_code ("} || '';");
@@ -395,7 +414,7 @@ sub _for
     $class->add_code ("my \$odd    = \$number % 2;");
     $class->add_code ("my \$even   = \$i % 2;");
     $class->add_code ("my \$start  = (\$i == 0);");
-    $class->add_code ("my \$end    = (\$i == \@array);");
+    $class->add_code ("my \$end    = (\$i == \$#array);");
     $class->add_code ("my \$inner  = (\$i and \$i < \@array);");
     
     # backwards compatibility
@@ -446,4 +465,14 @@ sub _tokenize
 }
 
 
+sub _decode_backslash_semicolon
+{
+    my $class = shift;
+    my $data  = shift;
+    
+    my $decode = new Petal::CodeGenerator::Decode ('xml');
+    return $decode->process ($data);
+}
+
+
 1;
